rem ---------------------------------------------------------------------------
rem  Trivadis AG, Baden-Dttwil/Basel/Bern/Lausanne/Zrich
rem               Dsseldorf/Frankfurt/Freiburg i.Br./Hamburg/Mnchen/Stuttgart
rem               Wien
rem               Switzerland/Germany/Austria Internet: http://www.trivadis.com
rem ---------------------------------------------------------------------------
rem ***************************************************************************
rem ****** sample script for course/seminar New Features 11g Release 1)  ******
rem ***************************************************************************
rem  Group/Privileges.: User
rem  Script-Name......: ex_execution_plan_stability.sql
rem  Developer........: Christian Antognini (christian.antognini@trivadis.com)
rem  Date.............: November 2007
rem  Version..........: Oracle Database 11g Release 1 - 11.1
rem  Description......: This script shows how capturing, selecting and evolving
rem                     of execution plan baselines works.
rem  Usage............: 
rem                     
rem  Input parameters.: 
rem  Output...........: 
rem  Called by........:
rem  Requirements.....: 
rem  Remarks..........: At the end the scripts deletes all baselines created
rem                     by the current user during the last 15 minutes.
rem
rem ---------------------------------------------------------------------------
rem Changes:
rem DD.MM.YYYY Developer Change
rem ---------------------------------------------------------------------------
rem
rem ---------------------------------------------------------------------------

SET ECHO ON TERMOUT ON SERVEROUTPUT OFF FEEDBACK OFF LONG 100000 PAGESIZE 100 LINESIZE 100

COLUMN PLAN_TABLE_OUTPUT FORMAT A80
COLUMN SQL_TEXT FORMAT A45
COLUMN SQL_HANDLE FORMAT A25

DROP TABLE t;

CLEAR SCREEN

REM
REM create test table and gather statistics
REM

CREATE TABLE t (id, n, pad, CONSTRAINT t_pk PRIMARY KEY (id))
AS 
SELECT rownum, rownum, rpad('*',500,'*')
FROM dba_objects WHERE rownum <= 1000;

BEGIN
  dbms_stats.gather_table_stats(
    ownname => user, 
    tabname => 't', 
    estimate_percent => 100, 
    method_opt => 'for all columns size 254',
    cascade => TRUE
  );
END;
/

PAUSE

REM
REM create a baseline (the query is executed twice because 
REM the first time it is not added to the baseline)
REM

ALTER SESSION SET optimizer_capture_sql_plan_baselines = TRUE;

SELECT count(pad) FROM t WHERE n = 42;

SELECT count(pad) FROM t WHERE n = 42;

ALTER SESSION SET optimizer_capture_sql_plan_baselines = FALSE;

PAUSE

SELECT sql_handle, sql_text, enabled, accepted FROM dba_sql_plan_baselines;

PAUSE

REM
REM display the execution plan stored in the baseline
REM

EXPLAIN PLAN FOR SELECT count(pad) FROM t WHERE n = 42;

SELECT * FROM table(dbms_xplan.display_sql_plan_baseline('&sql_handle', NULL, 'basic'));

PAUSE

REM
REM create an index to avoid the full table scan
REM

CREATE INDEX i ON t (n);

PAUSE

REM
REM the index is not used because the baseline is used
REM

EXPLAIN PLAN FOR SELECT count(pad) FROM t WHERE n = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic note'));

PAUSE

REM
REM execute twice the query... then the query optimizer notice that
REM another execution plan could be used to process the query
REM

SELECT count(pad) FROM t WHERE n = 42;

SELECT count(pad) FROM t WHERE n = 42;

PAUSE

REM
REM the new execution plan is added to the baseline but not accepted
REM

SELECT sql_handle, sql_text, enabled, accepted FROM dba_sql_plan_baselines;

PAUSE

REM
REM display the execution plans of the baseline
REM

SELECT * FROM table(dbms_xplan.display_sql_plan_baseline('&sql_handle', NULL, 'basic'));

PAUSE

REM
REM evolve the baseline 
REM

SELECT dbms_spm.evolve_sql_plan_baseline('&sql_handle') 
FROM dual;

PAUSE

SELECT sql_handle, sql_text, enabled, accepted FROM dba_sql_plan_baselines;

PAUSE

REM
REM check if the new execution plan is used
REM

EXPLAIN PLAN FOR SELECT count(pad) FROM t WHERE n = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic note'));

PAUSE

REM
REM what happens if the index is dropped?
REM

DROP INDEX i;

PAUSE

EXPLAIN PLAN FOR SELECT count(pad) FROM t WHERE n = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic note'));

PAUSE

REM
REM what happens if the index is recreated?
REM

CREATE INDEX i ON t (n);

PAUSE

EXPLAIN PLAN FOR SELECT count(pad) FROM t WHERE n = 42;

SELECT * FROM table(dbms_xplan.display(NULL, NULL, 'basic note'));

PAUSE

REM
REM remove all baselines created in the last 15 minutes
REM

DECLARE
  ret PLS_INTEGER;
BEGIN
  FOR c IN (SELECT DISTINCT sql_handle 
            FROM dba_sql_plan_baselines 
            WHERE creator = user
            AND created > systimestamp - to_dsinterval('0 00:15:00'))
  LOOP
    ret := dbms_spm.drop_sql_plan_baseline(c.sql_handle);
  END LOOP;
END;
/

SET ECHO OFF
